package style_Net_01;

import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;

public class Artifact {

	private static Grid <Object > artifactGrid;
	
	public static void setSpaceAndGrid (ContinuousSpace <Object > space , Grid <Object > grid) {
		artifactGrid = grid;
	}
		
	// INSTANCE VARIABLES 
	public int			artifactId;
	public LithicSource lithicRawMaterial;                
	public int          remainingUseLife;
	public GridPoint    discardLocation;
	public double       sourceDiscardDistance;  // distance from lithic source when discarded (km)
	public Person       owner;
	public static int nextArtifactId = 0;
	
	// CONSTRUCTOR
	public Artifact (Person o, LithicSource ls) {
		
		owner = o;
		lithicRawMaterial = ls;
		remainingUseLife = Model.getToolUseLife();          //give the new tool maximum use life
		artifactId = nextArtifactId++;
	}

	// ARTIFACT METHOD 1: Use Artifact
	public void use () {
		remainingUseLife = remainingUseLife - 1;                     //subtract 1 from the use-life
		if (remainingUseLife == 0) {                                 //if at zero
			discard();                                                   //call the method to discard
		}
	}
	
	// ARTIFACT METHOD 2: Discard Artifact
	public void discard() {
		
		discardLocation = owner.getCurrentLocation();
		
		//WARNING: This code assumes only a single lithic source!
		LithicSource lith = Model.getLithicSourceFromList(0); 
		
		GridPoint sourceLocation = artifactGrid.getLocation(lith); //get location of lithic source (our ground zero)
		sourceDiscardDistance = artifactGrid.getDistance (sourceLocation, discardLocation);
		sourceDiscardDistance = sourceDiscardDistance * 10;    //convert to km
		owner.removeArtifactFromList(this);                      //send to owner to remove from list
	}
	
	// SETTERS AND GETTERS
	public LithicSource getLithicRawMaterial() { return lithicRawMaterial; }
	public void setLithicRawMaterial( LithicSource ls) { lithicRawMaterial = ls; }
	
	public int getLithicSourceId() { 
		int lsid = lithicRawMaterial.getLithicSourceId();
		return lsid;
	}
	
	public int getRemainingUseLife() { return remainingUseLife; }

	public double getSourceDiscardDistance() { return sourceDiscardDistance; }
	public void setSourceDiscardDistance(double x) {sourceDiscardDistance = x; }
	
	public void setOwner(Person ow) {owner = ow;}
	public Person getOwner() {return owner;}
	
	public int getArtifactId() {return artifactId; }
	
}
