package style_Net_01;

import java.util.ArrayList;
import java.util.List;

import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.SimUtilities;

public class Group {
	
	private ContinuousSpace <Object > space;
	private Grid <Object > grid;
	
	// INSTANCE VARIABLES
	public int 			groupId;
	public int          x;
	public int          y;
	public AggSite      aggSite;
	public ArrayList<Person>      	groupMemberList = new ArrayList <Person> ();     // list of people    
	
	// CONSTRUCTOR
	public Group (int gid, AggSite as) {
			groupId = gid;
			aggSite = as;
	}
	
	public void setGroupSpaceAndGrid (ContinuousSpace <Object > sentSpace , Grid <Object > sentGrid) {
		space = sentSpace;
		grid = sentGrid;
	}
	
	// GROUP METHOD 1: Add a Person to the Group
	public void addPersonToGroup (Person addPerson) {
		groupMemberList.add (addPerson);
		addPerson.setX(x);
		addPerson.setY(y);
	}
	
	// GROUP METHOD 2: Remove a Person from the Group
	public void removePersonFromGroup (Person removePerson) {
		groupMemberList.remove(removePerson);
	}
	
	// GROUP METHOD 3: Move Away from Aggregation Site
	public void moveAwayFromAggSite (int moveDistance) {
	
		GridPoint aggPointGrid = grid.getLocation(aggSite);       //get location of aggSite
		GridPoint currentLocationGrid = grid.getLocation(this);    //get group's location
		double groupDistanceFromAgg = grid.getDistance(aggPointGrid, currentLocationGrid);        
		
		GridCellNgh <Parcel> allParcels = new GridCellNgh <Parcel>( grid, currentLocationGrid, Parcel.class, moveDistance, moveDistance);
		List <GridCell <Parcel> > allNeighborParcels = allParcels.getNeighborhood ( false);
		
		GridPoint movePoint = null;
		ArrayList <GridCell <Parcel>> moveParcels = new ArrayList <GridCell <Parcel>>(); //list to hold potential parcels
		
		for ( GridCell < Parcel > cell : allNeighborParcels ) {            //go through list of all
			GridPoint cellPointGrid = cell.getPoint();
			double cellDistanceFromAgg = grid.getDistance (aggPointGrid, cellPointGrid);   //get distance of this cell from the agg site
			if (cellDistanceFromAgg > groupDistanceFromAgg) {              //if a move would place us farther from agg
				if (cell.size() == 0) {							//if cell is unoccupied
					moveParcels.add(cell);                            //add to list
				}
			}
		}
		
		SimUtilities.shuffle (moveParcels, RandomHelper.getUniform());          //shuffle the list
		
		GridCell <Parcel> moveToCell = moveParcels.get(0);          //get the first one on the list
		movePoint = moveToCell.getPoint();                           //get the point to move to
		
		x = movePoint.getX();
		y = movePoint.getY();
		
		space.moveTo(this, x, y);
		grid.moveTo(this, x, y);
		
		for (Person groupPerson : groupMemberList) {
			groupPerson.moveWithGroup (x, y);
		}
		
		if (Model.getTimePeriod() != 0) {  //if we're recording data
			double maxRangeKm = Model.getMaxRangeKm();
		
			currentLocationGrid = grid.getLocation(this);    //get group's location
			groupDistanceFromAgg = grid.getDistance(aggPointGrid, currentLocationGrid);      
			
			if (groupDistanceFromAgg * 10 > maxRangeKm) {       //if this is the farthest moved yet
				Model.setMaxRangeKm(groupDistanceFromAgg*10);         //record it
			}
		}
	}

	// GROUP METHOD 4: Move Toward Aggregation Site
	public void moveTowardAggSite (int moveDistance) {
		GridPoint aggPointGrid = grid.getLocation(aggSite);       //get location of aggSite
		GridPoint currentLocationGrid = grid.getLocation(this);    //get group's location
		
		if (aggPointGrid != currentLocationGrid) {                    //if group is NOT already at AggSite
			
			double groupDistanceFromAgg = grid.getDistance (aggPointGrid, currentLocationGrid);   //get distance of this one from the agg site
			
			if (groupDistanceFromAgg < moveDistance) {                   //if we're within move distance
				moveToAggSite();                                      //call method to move directly to Agg Site
			}		
			
			if (groupDistanceFromAgg == moveDistance) {  
				moveToAggSite();
			}
			
			if (Model.getSeasonTick() == 52) {  //if it's the last week of the year
				moveToAggSite();                //get your asses back to the agg site
			}
			
			else { 	//if we're still afield, this moves the group closer
					GridCellNgh <Parcel> allParcels = new GridCellNgh <Parcel>( grid, currentLocationGrid, Parcel.class, moveDistance, moveDistance);
					List <GridCell <Parcel> > allNeighborParcels = allParcels.getNeighborhood ( false);
					
					GridPoint movePoint = null;                //the point to move to   
					ArrayList <GridCell <Parcel>> moveParcels = new ArrayList <GridCell <Parcel>>();   //list to hold possible move points
			
					for ( GridCell < Parcel > cell : allNeighborParcels ) {            //go through list of all
						GridPoint cellPointGrid = cell.getPoint();                        //for each cell
						double cellDistanceFromAgg = grid.getDistance (aggPointGrid, cellPointGrid);   //get distance of this one from the agg site
							
						if (cellDistanceFromAgg < groupDistanceFromAgg) {              //if a move would place us closer to the aggSite
							if (cell.size() == 0) {							//and if cell is unoccupied
								moveParcels.add(cell);                            //add to list
							}
						}
					}

					SimUtilities.shuffle (moveParcels, RandomHelper.getUniform());          //shuffle the list

				if (moveParcels.size() != 0) {   //if list is not empty
					GridCell <Parcel> moveToCell = moveParcels.get(0);          //get the first one on the list
					movePoint = moveToCell.getPoint();                           //get the point to move to

					x = movePoint.getX();
					y = movePoint.getY();

					space.moveTo(this, x, y);
					grid.moveTo(this, x, y);
			
					for (Person groupPerson : groupMemberList) {
						groupPerson.moveWithGroup (x, y);
					}
				}
			}
		}	
	}

	// GROUP METHOD 5: Move to Aggregation Site
	public void moveToAggSite() {
		int aggX = aggSite.getAggX();
		int aggY = aggSite.getAggY();
		
		space.moveTo(this, aggX, aggY);
		grid.moveTo(this, aggX, aggY);
		
		for (Person groupPerson : groupMemberList) {
			groupPerson.moveWithGroup (aggX, aggY);
		}
	}
		
	public int getNumMembers () {
		return groupMemberList.size();
	}
	
	public int getX() {return x;}
	public void setX (int i) { x = i; }
	
	public int getY() {return y;}
	public void setY (int i) { y = i; }
	
	public int getSize() {return groupMemberList.size();}

	public Person getMember(int i) {return groupMemberList.get(i); }
	
	public AggSite getAggSite() {return aggSite; }
	
	public int getGroupId() {return groupId; }
	
	public GridPoint getCurrentLocation () {
		GridPoint currentLocationGrid = grid.getLocation(this);    //get group's location
		return currentLocationGrid;
	}
}
