package style_Net_01;

import java.util.ArrayList;
import java.util.List;

import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.SimUtilities;

public class Person {

	private ContinuousSpace<Object> space;
	private Grid<Object> grid;

	public void setPersonSpaceAndGrid(ContinuousSpace<Object> sentSpace, Grid<Object> sentGrid) {
		space = sentSpace;
		grid = sentGrid;
	}

	// Instance Variables
	public int id; // unique id number for each person
	public int x;
	public int y;
	public AggSite aggSite; // aggregation site to which person reports
	public Group group; // group that person is in

	public static int nextId = 0;

	public ArrayList<Artifact> toolList = new ArrayList<Artifact>(); // list tools
	
	public LithicSource proximateLithicSource;

	public Person() {
		aggSite = null;
		id = nextId++;
	}

	//PERSON METHOD 1: Move with Group
	public void moveWithGroup(int groupX, int groupY) {
		x = groupX;
		y = groupY;

		space.moveTo(this, x, y);
		grid.moveTo(this, x, y);
		
		GridPoint groupLocation = group.getCurrentLocation();
		GridPoint personLocation = grid.getLocation(this);
		
		if (groupLocation.getX() != personLocation.getX()) {
			System.out.printf ("Error: person not in same cell as group: x different\n");
		}
		
		if (groupLocation.getY() != personLocation.getY()) {
			System.out.printf ("Error: person not in same cell as group: y different\n");
		}
	}

	//PERSON METHOD 2: Add Tool to List
	public void addToolToList(Artifact newTool) {
		toolList.add(newTool);
	}

	//PERSON METHOD 3: Check Lithic Source Proximity
	public void checkLithicSourceProximity() {
	
		proximateLithicSource = null;    //set to null each time before search
		
		GridPoint personLocation = grid.getLocation(this); // person's location
		int pmr = Model.getPersonalMobilityRadius(); // personal mobility radius
		
		ArrayList<LithicSource> nearbyLithicSources = new ArrayList<LithicSource>(); // list to store nearby lithic sources
		
		GridCellNgh<LithicSource> neighborhood = new GridCellNgh<LithicSource>(grid, personLocation, LithicSource.class, pmr, pmr);
				
		List<GridCell<LithicSource>> nearSource = neighborhood.getNeighborhood(true);  
		
		for (GridCell<LithicSource> cell : nearSource) { // for each cell on the list
			if (cell.size() > 0) { // if cell is not empty
				List<LithicSource> nearSources = (List<LithicSource>) cell.items(); // list what's in it
				for (LithicSource source : nearSources) { // go through that list
					nearbyLithicSources.add(source); // add to source list}
				}
			}
		}
		
		if (nearbyLithicSources.size() != 0) {          //if there are sources on the list
			proximateLithicSource = nearbyLithicSources.get(0);  //get the first one on the list, for now we'll assume there is just one source per list
		}
	}
	
	//PERSON METHOD 4: Discard Used Tools
	public void discardUsedTools() {
		int useLife = Model.getToolUseLife();
		ArrayList<Artifact> discardToolList = new ArrayList<Artifact>(); // list tools to discard
	
		for (Artifact tool : toolList) {    //go through toolList
			if (tool.getRemainingUseLife() < useLife)  //if it's been used
				discardToolList.add(tool);      //add it to list
		}
		
		for (Artifact tool : discardToolList) {  //now go through this list and discard them all
			tool.discard();           //discard it
		}
	}
		
	//PERSON METHOD 5: Create Tools
	public void createTools() {
		int inventorySize = Model.getToolInventorySize(); 
		int pf = Model.getProductionFactor();
		
		int numToolsToCreate = inventorySize - toolList.size(); 
	
		if (proximateLithicSource != null) {  //if we're near a source
			inventorySize = inventorySize * pf;   //recalculate the inventory size
			numToolsToCreate = inventorySize - toolList.size();  //recalculate number of tools to create with production factor
		}
		
		if (numToolsToCreate != 0) { // if we need to make tools
			for (int i = 0; i < numToolsToCreate; ++i) {        //loop to make enough tools\\
				Artifact newTool = new Artifact(this, proximateLithicSource); 
				toolList.add(newTool); // add to person's tool inventory
			}
		}		
	}

	//PERSON METHOD 6: Use Tools
	public void useTools() {
		int toolUsesPerStep = Model.getToolUsesPerStep(); // get number of uses per user per step
		for (int i = 0; i < toolUsesPerStep; ++i) { // loop to use
			
			int usesToPerform = toolUsesPerStep - i;  //calculate number of uses left to go
			int invUseLifeRemaining = 0;
			
			for (Artifact tool : toolList) {       //go through toolList
				invUseLifeRemaining = invUseLifeRemaining + tool.getRemainingUseLife(); //add remaining useLife to tally
			}
			
			if (invUseLifeRemaining < usesToPerform) {        //if we don't have enough tools to do all the uses
				createTools();             //hit the create tools loop again
			}
			
			int chosenArtifact = RandomHelper.nextIntFromTo(0, toolList.size() - 1); // choose a random tool
			Artifact artifactToUse = toolList.get(chosenArtifact); // retrieve the tool
			artifactToUse.use(); // call method to subtract a use from the artifact's use-life
		}
	}

	//PERSON METHOD 7: Remove Artifact from List
	public void removeArtifactFromList(Artifact removeArtifact) {
		toolList.remove(removeArtifact);
		
		if (Model.getTimePeriod() != 0) {  //if we're in T1 or T2
			int roll = RandomHelper.nextIntFromTo(1, 2);
				if (roll == 1) {                    //50% sample
					Model.recordArtifact(removeArtifact);
				}
		}
		
		else {								//if not recording it
			removeArtifact = null;  		//null it
		}
	}

	//PERSON METHOD 8: Change Groups (not used in this iteration of the model)
	public void changeGroups(int pmr) { // pmr = personalMobilityRadius

		ArrayList<Group> exchangeGroups = new ArrayList<Group>(); 

		GridPoint personLocation = grid.getLocation(this); // person's location

		GridCellNgh<Group> neighborhood = new GridCellNgh<Group>(grid, personLocation, Group.class, pmr, pmr);
		List<GridCell<Group>> neighborCells = neighborhood.getNeighborhood(true);

		for (GridCell<Group> cell : neighborCells) { // for each cell on the
														// list
			if (cell.size() > 0) { // if cell is not empty
				List<Group> nearGroups = (List<Group>) cell.items(); // list
																		// what's
																		// in it
				for (Group nearGroup : nearGroups) { // go through that list
					if (nearGroup != group) { // if group is a different group
						if (nearGroup.getSize() < Model.getMaxGroupSize()) { // and
																				// if
																				// group
																				// isn't
																				// maxed
																				// out
							exchangeGroups.add(nearGroup); // add to exchange
															// list
						}
					}
				}
			}
		}

		if (exchangeGroups.size() != 0) { // if there are groups nearby
			SimUtilities.shuffle(exchangeGroups, RandomHelper.getUniform()); // shuffle
																				// list
			Group moveGroup = exchangeGroups.get(0); // get first one on list

			if (moveGroup.getAggSite() == aggSite) { // if we're from same Agg
														// site

				group.removePersonFromGroup(this); // remove person from current
													// group

				moveGroup.addPersonToGroup(this); // add person to new group
				group = moveGroup; // set person's group to new group
				
				if (Model.getTimePeriod() != 0) {  //if we're recording data
					Model.incrementNumInterGroupMoves();
				}

			}

			if (moveGroup.getAggSite() != aggSite) { // if from different Agg
														// site

				double roll = RandomHelper.nextDoubleFromTo(0, 1);
				double piam = Model.getPInterAggMovement();
				if (roll < piam) {

				//	System.out.printf("moving to group from different Agg site\n");

					moveGroup.addPersonToGroup(this);
					group.removePersonFromGroup(this);
					group = moveGroup; // switch groups
					aggSite.removePersonFromPopulationList (this);  //remove from current agg site
					aggSite = moveGroup.getAggSite(); // switch agg sites
					aggSite.addPersonToPopulationList(this);  //add person to new agg site
			
					if (Model.getTimePeriod() != 0) {  //if we're recording data
						Model.incrementNumInterGroupMoves();
						Model.incrementNumInterAggMoves();
					}
				}
			}
		}
	}

	//SETTERS AND GETTERS
	public int getId() {
		return id;
	}

	public int getX() {
		return x;
	}

	public void setX(int i) {
		x = i;
	}

	public int getY() {
		return y;
	}

	public void setY(int i) {
		y = i;
	}

	public GridPoint getCurrentLocation() {
			GridPoint currentLocation = grid.getLocation(this);
			return currentLocation;
	}
	
	public AggSite getAggSite() {
		return aggSite;
	}

	public void setAggSite(AggSite as) {
		aggSite = as;
	}

	public int getAggSiteNo() {
		return aggSite.getAggId();
	}

	public Group getGroup() {
		return group;
	}
	
	public LithicSource getProximateLithicSource() {
		return proximateLithicSource;
	}

	public void setGroup(Group personGroup) {
		group = personGroup;
	}

	public int getNumTools() {
		return toolList.size();
	} // return size of tool list

	public Artifact getTool(int t) {
		return toolList.get(t);
	} // get a tool from the list

}
