package style_Net_01;


import repast.simphony.context.Context;
import repast.simphony.context.space.continuous.ContinuousSpaceFactory;
import repast.simphony.context.space.continuous.ContinuousSpaceFactoryFinder;
import repast.simphony.context.space.grid.GridFactory;
import repast.simphony.context.space.grid.GridFactoryFinder;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.RandomCartesianAdder;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridBuilderParameters;
import repast.simphony.space.grid.SimpleGridAdder;
import repast.simphony.space.grid.StrictBorders;

public class Style_Net_01Builder implements ContextBuilder<Object> {
	
	public Model newModel = new Model (); //create a model
	
	public static int runNumber = 0;
	
	public static Context<Object> context = null;     //create a null context
	public static ContinuousSpace<Object> space = null;
	public static Grid<Object> grid = null;
	
	@Override
	public Context build(Context<Object> context) {
		
		++runNumber; 
	
		context.setId ("stylenet");
		
		Model.setupParams();  //call this first to establish parameters, clear lists, etc.
		
		int sizeX = Model.getSizeX();
		int sizeY = Model.getSizeY();
		
		ContinuousSpaceFactory spaceFactory =
		ContinuousSpaceFactoryFinder.createContinuousSpaceFactory ( null );
		 
		ContinuousSpace < Object > space =
		spaceFactory.createContinuousSpace ("space", context, new RandomCartesianAdder < Object >() ,
		new repast.simphony.space.continuous.WrapAroundBorders () ,
		sizeX , sizeY);
	
		GridFactory gridFactory = GridFactoryFinder.createGridFactory ( null );
			Grid <Object > grid = gridFactory.createGrid ("grid", context,
			new GridBuilderParameters <Object >( new StrictBorders (),
			new SimpleGridAdder <Object >(), true, sizeX, sizeY));
		 
		Model.setSpaceAndGrid(space, grid);  //send the space and grid to the instance of model	
		Artifact.setSpaceAndGrid (space, grid);
		
		//BUILDER METHOD 1: Create Aggregation Sites
		int numAggSites = Model.getNumAggSites();
		if (numAggSites == 1) {              //if we're creating just one
			Model.createAggSites();         //tell model to create it
		}
		else {
			System.out.printf("ERROR: Number of aggregation sites != 1\n");
		}
				
		//Loop to add agg sites to context
		for (int i=0; i < numAggSites; ++i ) {
			AggSite addAgg = Model.getAggSiteFromList(i);
			context.add(addAgg);
			int aggX = addAgg.getAggX();
			int aggY = addAgg.getAggY();
			space.moveTo(addAgg, aggX, aggY);
			grid.moveTo(addAgg, aggX, aggY);
		}
		
		//BUILDER METHOD 2: Create Lithic Sources
		int numLithicSources = Model.getNumLithicSources();
		if (numLithicSources == 1) {        //if we're creating just one
			Model.createLithicSources();
		}
		else {
			System.out.printf("ERROR: Number of lithic sources != 1\n");
		}
		
		//Loop to add lithic sources to context
		for (int i=0; i < numLithicSources; ++i) {
			LithicSource addLithicSource = Model.getLithicSourceFromList(i);
			context.add (addLithicSource);
			int lsX = addLithicSource.getLithicSourceX();
			int lsY = addLithicSource.getLithicSourceY();
			space.moveTo(addLithicSource,  lsX, lsY);
			grid.moveTo(addLithicSource,  lsX, lsY);
		}
		
		//BUILDER METHOD 3: Create People and Assign to Groups
		Model.createPeople();           //tell model to create people
		Model.assignPeopleToAggSites(); //tell model to put people into agg sites
		Model.assignPeopleToGroups();   //tell model to put people into groups
		
		//Loop to add people to context
		int numPeople = Model.getNumPersonsPerAggSite() * Model.getNumAggSites();
		
		for (int i=0; i < numPeople; ++i ) {
			Person addPerson = Model.getPersonFromList(i);
			context.add(addPerson);
			addPerson.setPersonSpaceAndGrid(space, grid);
			int personX = addPerson.getX();
			int personY = addPerson.getY();
			space.moveTo(addPerson, personX, personY);
			grid.moveTo(addPerson, personX, personY);
		}
		
		//loop to add groups to context
		int numGroups = Model.groupList.size();
					
		for (int i=0; i < numGroups; ++i ) {
			Group addGroup = Model.getGroupFromList(i);
			context.add(addGroup);
			int groupX = addGroup.getX();
			int groupY = addGroup.getY();
			space.moveTo(addGroup, groupX, groupY);
			grid.moveTo(addGroup, groupX, groupY);
			addGroup.setGroupSpaceAndGrid (space, grid);
		}
		
		context.add(newModel); //add the model to the context
		
		return context ;
	 }
	
public static void nullContext() {
		
		context = null;
		space = null;
		grid = null;
	}
	
	public static int getRunNumber () {	return runNumber; }
}
